@@ -61,6 +61,11 @@ class TestHandler(urllib.request.HTTPHandler, urllib.request.HTTPSHandler):
     request = None
     site = ''
 
+    def __init__(self, debuglevel=0, **kwargs):
+        self._debuglevel = debuglevel
+        self._context = kwargs.get('context')
+        self._check_hostname = kwargs.get('check_hostname')
+
     @classmethod
     def set_response(cls, response):
         """Set a static response to be returned for all requests.
